import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32inet import *
@pytest.mark.xfail
def test_CommitUrlCacheEntry():
    CommitUrlCacheEntry()     



@pytest.mark.xfail
def test_CreateUrlCacheEntry():
    CreateUrlCacheEntry()     



def test_CreateUrlCacheGroup():
    CreateUrlCacheGroup()     



@pytest.mark.xfail
def test_DeleteUrlCacheEntry():
    DeleteUrlCacheEntry()     



@pytest.mark.xfail
def test_DeleteUrlCacheGroup():
    DeleteUrlCacheGroup()     



@pytest.mark.xfail
def test_FindCloseUrlCache():
    FindCloseUrlCache()     



@pytest.mark.xfail
def test_FindFirstUrlCacheEntry():
    FindFirstUrlCacheEntry()     



@pytest.mark.xfail
def test_FindFirstUrlCacheEntryEx():
    FindFirstUrlCacheEntryEx()     



def test_FindFirstUrlCacheGroup():
    FindFirstUrlCacheGroup()     



@pytest.mark.xfail
def test_FindNextUrlCacheEntry():
    FindNextUrlCacheEntry()     



@pytest.mark.xfail
def test_FindNextUrlCacheEntryEx():
    FindNextUrlCacheEntryEx()     



@pytest.mark.xfail
def test_FindNextUrlCacheGroup():
    FindNextUrlCacheGroup()     



@pytest.mark.xfail
def test_FtpCommand():
    FtpCommand()     



@pytest.mark.xfail
def test_FtpOpenFile():
    FtpOpenFile()     



@pytest.mark.xfail
def test_GetUrlCacheEntryInfo():
    GetUrlCacheEntryInfo()     



@pytest.mark.xfail
def test_GetUrlCacheGroupAttribute():
    GetUrlCacheGroupAttribute()     



def test_InternetAttemptConnect():
    InternetAttemptConnect()     



@pytest.mark.xfail
def test_InternetCanonicalizeUrl():
    InternetCanonicalizeUrl()     



@pytest.mark.xfail
def test_InternetCheckConnection():
    InternetCheckConnection()     



@pytest.mark.xfail
@pytest.mark.xfail
def test_InternetCloseHandle():
    InternetCloseHandle()     



@pytest.mark.xfail
def test_InternetConnect():
    InternetConnect()     



@pytest.mark.xfail
@pytest.mark.xfail
def test_InternetGetCookie():
    InternetGetCookie()     



@pytest.mark.xfail
def test_InternetGetLastResponseInfo():
    InternetGetLastResponseInfo()     



@pytest.mark.xfail
def test_InternetGoOnline():
    InternetGoOnline()     



@pytest.mark.xfail
def test_InternetOpen():
    InternetOpen()     



@pytest.mark.xfail
def test_InternetOpenUrl():
    InternetOpenUrl()     



@pytest.mark.xfail
def test_InternetQueryOption():
    InternetQueryOption()     



@pytest.mark.xfail
def test_InternetReadFile():
    InternetReadFile()     



@pytest.mark.xfail
def test_InternetSetCookie():
    InternetSetCookie()     



@pytest.mark.xfail
def test_InternetSetOption():
    InternetSetOption()     



@pytest.mark.xfail
def test_InternetWriteFile():
    InternetWriteFile()     



@pytest.mark.xfail
def test_SetUrlCacheEntryGroup():
    SetUrlCacheEntryGroup()     



@pytest.mark.xfail
def test_SetUrlCacheGroupAttribute():
    SetUrlCacheGroupAttribute()     





import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from adsi import *
@pytest.mark.xfail
def test_ADsBuildEnumerator():
    ADsBuildEnumerator()     



@pytest.mark.xfail
def test_ADsEnumerateNext():
    ADsEnumerateNext()     



def test_ADsGetLastError():
    ADsGetLastError()     



@pytest.mark.xfail
def test_ADsGetObject():
    ADsGetObject()     



@pytest.mark.xfail
def test_ADsOpenObject():
    ADsOpenObject()     



@pytest.mark.xfail
def test_DSOP_SCOPE_INIT_INFOs():
    DSOP_SCOPE_INIT_INFOs()     



@pytest.mark.xfail
def test_StringAsDS_SELECTION_LIST():
    StringAsDS_SELECTION_LIST()     





import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32pdh import *
@pytest.mark.xfail
def test_AddCounter():
    AddCounter()     



@pytest.mark.xfail
def test_BrowseCounters():
    BrowseCounters()     



@pytest.mark.xfail
def test_CloseQuery():
    CloseQuery()     



@pytest.mark.xfail
def test_CollectQueryData():
    CollectQueryData()     



@pytest.mark.xfail
def test_ConnectMachine():
    ConnectMachine()     



@pytest.mark.xfail
def test_EnumObjectItems():
    EnumObjectItems()     



@pytest.mark.xfail
def test_EnumObjects():
    EnumObjects()     



@pytest.mark.xfail
def test_ExpandCounterPath():
    ExpandCounterPath()     



@pytest.mark.xfail
def test_GetCounterInfo():
    GetCounterInfo()     



@pytest.mark.xfail
def test_GetFormattedCounterValue():
    GetFormattedCounterValue()     



@pytest.mark.xfail
def test_LookupPerfIndexByName():
    LookupPerfIndexByName()     



@pytest.mark.xfail
def test_LookupPerfNameByIndex():
    LookupPerfNameByIndex()     



@pytest.mark.xfail
def test_MakeCounterPath():
    MakeCounterPath()     



def test_OpenQuery():
    OpenQuery()     



@pytest.mark.xfail
def test_ParseCounterPath():
    ParseCounterPath()     



@pytest.mark.xfail
def test_ParseInstanceName():
    ParseInstanceName()     



@pytest.mark.xfail
def test_RemoveCounter():
    RemoveCounter()     



@pytest.mark.xfail
def test_SetCounterScaleFactor():
    SetCounterScaleFactor()     



@pytest.mark.xfail
def test_ValidatePath():
    ValidatePath()     





import os
import yaml
import glob
import shutil
import subprocess
import importlib
import tempfile

from toposort import toposort_flatten
from collections import deque
from contextlib import suppress
from copy import copy
from distutils._msvccompiler import MSVCCompiler, _find_exe
from .extension import WinExt


def find_directories(parent='.'):
    """Recursively find directories that contain a module.yml

    No directories should be returned that have a parent
    containing module.yml
    """
    # FIXME: This implementation does not comply with the second point

    for fname in glob.iglob(
            os.path.join(parent, '**/module.yml'), recursive=True):
        yield os.path.relpath(os.path.dirname(fname), parent)


def collect_extensions(parent='.'):
    """Collect all directories that contain a 'module.yml' recursively.

    The 'include' directory of each module is added by default to the include
    path, and 'include' directories from other extensions are added to the 
    include path if they are listed in the 'modules'
    """

    # This is somewhat confusing, because the 'modules' key in the yaml
    # files specifies the dependencies for that module, the the 'modules'
    # variable contains the actual module configurations.
    modules = {}
    dependencies = {}
    build_order = ('.i', '.mc', '.rc', '.cpp')

    for dir in find_directories(parent):
        yml = os.path.join(parent, dir, 'module.yml')
        with open(yml) as fh:
            cfg = yaml.load(fh)

        if yml is None:
            raise ValueError('Invalid configuration file')

        name = dir.replace(os.sep, '.')
        modules[name] = cfg  # Module configuration
        dependencies[name] = set(cfg.setdefault('modules',
                                                []))  # Module dependencies

    external_modules = []
    for name in toposort_flatten(dependencies):
        src = os.path.join(parent, name.replace('.', os.sep), 'src')
        sources = []

        try:
            files = os.listdir(src)
        except FileNotFoundError:
            external_modules += [
                name
            ]  # Add this to external_modules for later generation
            continue

        for item in build_order:  # Add all sources present in the 'src' directory
            for fname in files:
                if fname.endswith(item):
                    sources.append(os.path.join(src, fname))

        include_dirs = []
        f = set()
        q = deque([name])
        i = 0

        # Simple dependency resolver algorithm:
        while q:
            i += 1
            assert i < 500  # To avoid infinite loop

            dep = q.popleft().replace('.', os.sep)  # Take one module
            include_dirs.extend([  # Add the include directories
                os.path.join(parent, dep, 'src'),
                os.path.join(parent, dep, 'include')
            ])
            f.add(dep.replace(os.sep,
                              '.'))  # Add the module's dependencies to the set
            q.extend([d for d in dependencies[name] if d not in f
                      ])  # Queue modules not already in the set for processing

        cfg = modules[name]
        del cfg['modules']  # Remove the 'modules' (depenencies) key

        with suppress(KeyError):
            for i, depend in enumerate(cfg['depends']):
                cfg['depends'][i] = os.path.join(name, 'include', depend)

        for config_option in ['export_symbol_file', 'pch_header']:
            with suppress(KeyError):
                cfg[config_option] = os.path.join(
                    name.replace('.', os.sep), 'include', cfg[config_option])

        build_temp_library_dirs = [
            os.path.join(os.path.dirname(d), 'src') for d in include_dirs
        ]

        yield WinExt(
            name,
            sources=sources,
            include_dirs=copy(include_dirs),
            external_modules=copy(external_modules),
            build_temp_library_dirs=list(set(build_temp_library_dirs)),
            **cfg)

        print('collected: {}'.format(name))


def find_tools():
    """Find and return the location of the 'lib' and 'dumpbin' tools.
    
    Also returns the environment in which they should be run.
    """
    compiler = MSVCCompiler()
    compiler.initialize()

    paths = compiler._paths.split(os.pathsep)
    lib = compiler.lib
    dumpbin = _find_exe('dumpbin.exe', paths)

    env = copy(os.environ)
    env['path'] = os.pathsep.join(paths)

    return lib, dumpbin, env


def generate_libs(libs, plat_name):
    """Generate lib files to link against from python extensions."""
    tmpdir = tempfile.mkdtemp()
    lib, dumpbin, env = find_tools()

    for module in iter(importlib.import_module(lib) for lib in libs):
        pyd = module.__file__
        basename, ext = os.path.splitext(os.path.basename(pyd))

        assert ext == '.pyd'

        dll = os.path.join(tmpdir, '{}.dll'.format(basename))
        dname = os.path.join(tmpdir, '{}.def'.format(basename))
        lname = os.path.join(tmpdir, '{}.lib'.format(basename))

        shutil.copy(pyd, dll)

        definitions = open(dname, 'w+')
        definitions.write('EXPORTS\n')
        dump = iter(
            subprocess.check_output([dumpbin, '/exports', dll], env=env)
            .decode().splitlines())

        while True:
            if next(dump).strip().split() == [
                    'ordinal', 'hint', 'RVA', 'name'
            ]:
                break

        for line in dump:
            if line.strip():
                if line.strip() == 'Summary':
                    break
                else:
                    definitions.write('{}\n'.format(line.strip().split()[-1]))

        definitions.close()
        if 'amd64' in plat_name:
            specifier = '/MACHINE:X64'
        else:
            specifier = '/MACHINE:X86'

        subprocess.call(
            [lib, '/def:{}'.format(dname), '/OUT:{}'.format(lname), specifier],
            env=env)

        yield lname



import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32file import *
@pytest.mark.xfail
def test_AcceptEx():
    AcceptEx()     



@pytest.mark.xfail
def test_AddUsersToEncryptedFile():
    AddUsersToEncryptedFile()     



@pytest.mark.xfail
def test_AllocateReadBuffer():
    AllocateReadBuffer()     



def test_AreFileApisANSI():
    AreFileApisANSI()     



@pytest.mark.xfail
def test_BackupRead():
    BackupRead()     



@pytest.mark.xfail
def test_BackupSeek():
    BackupSeek()     



@pytest.mark.xfail
def test_BackupWrite():
    BackupWrite()     



@pytest.mark.xfail
def test_BuildCommDCB():
    BuildCommDCB()     



@pytest.mark.xfail
def test_CalculateSocketEndPointSize():
    CalculateSocketEndPointSize()     



@pytest.mark.xfail
def test_CancelIo():
    CancelIo()     



@pytest.mark.xfail
def test_ClearCommBreak():
    ClearCommBreak()     



@pytest.mark.xfail
def test_ClearCommError():
    ClearCommError()     



@pytest.mark.xfail
def test_CloseEncryptedFileRaw():
    CloseEncryptedFileRaw()     



@pytest.mark.xfail
def test_CloseHandle():
    CloseHandle()     



@pytest.mark.xfail
def test_ConnectEx():
    ConnectEx()     



@pytest.mark.xfail
def test_CopyFile():
    CopyFile()     



@pytest.mark.xfail
def test_CopyFileEx():
    CopyFileEx()     



@pytest.mark.xfail
def test_CopyFileW():
    CopyFileW()     



@pytest.mark.xfail
def test_CreateDirectory():
    CreateDirectory()     



@pytest.mark.xfail
def test_CreateDirectoryEx():
    CreateDirectoryEx()     



@pytest.mark.xfail
def test_CreateDirectoryExW():
    CreateDirectoryExW()     



@pytest.mark.xfail
def test_CreateDirectoryW():
    CreateDirectoryW()     



@pytest.mark.xfail
def test_CreateFile():
    CreateFile()     



@pytest.mark.xfail
def test_CreateFileW():
    CreateFileW()     



@pytest.mark.xfail
def test_CreateHardLink():
    CreateHardLink()     



@pytest.mark.xfail
def test_CreateIoCompletionPort():
    CreateIoCompletionPort()     



@pytest.mark.xfail
def test_CreateMailslot():
    CreateMailslot()     



@pytest.mark.xfail
def test_CreateSymbolicLink():
    CreateSymbolicLink()     



@pytest.mark.xfail
def test_DecryptFile():
    DecryptFile()     



@pytest.mark.xfail
def test_DefineDosDevice():
    DefineDosDevice()     



@pytest.mark.xfail
def test_DefineDosDeviceW():
    DefineDosDeviceW()     



@pytest.mark.xfail
def test_DeleteFile():
    DeleteFile()     



@pytest.mark.xfail
def test_DeleteFileW():
    DeleteFileW()     



@pytest.mark.xfail
def test_DeleteVolumeMountPoint():
    DeleteVolumeMountPoint()     



@pytest.mark.xfail
def test_DeviceIoControl():
    DeviceIoControl()     



@pytest.mark.xfail
def test_DuplicateEncryptionInfoFile():
    DuplicateEncryptionInfoFile()     



@pytest.mark.xfail
def test_EncryptFile():
    EncryptFile()     



@pytest.mark.xfail
def test_EncryptionDisable():
    EncryptionDisable()     



@pytest.mark.xfail
def test_EscapeCommFunction():
    EscapeCommFunction()     



@pytest.mark.xfail
def test_FILE_NOTIFY_INFORMATION():
    FILE_NOTIFY_INFORMATION()     



@pytest.mark.xfail
def test_FileEncryptionStatus():
    FileEncryptionStatus()     



@pytest.mark.xfail
def test_FindClose():
    FindClose()     



@pytest.mark.xfail
def test_FindCloseChangeNotification():
    FindCloseChangeNotification()     



@pytest.mark.xfail
def test_FindFileNames():
    FindFileNames()     



@pytest.mark.xfail
def test_FindFilesIterator():
    FindFilesIterator()     



@pytest.mark.xfail
def test_FindFilesW():
    FindFilesW()     



@pytest.mark.xfail
def test_FindFirstChangeNotification():
    FindFirstChangeNotification()     



@pytest.mark.xfail
def test_FindNextChangeNotification():
    FindNextChangeNotification()     



@pytest.mark.xfail
def test_FindStreams():
    FindStreams()     



@pytest.mark.xfail
def test_FlushFileBuffers():
    FlushFileBuffers()     



@pytest.mark.xfail
def test_GetAcceptExSockaddrs():
    GetAcceptExSockaddrs()     



@pytest.mark.xfail
def test_GetBinaryType():
    GetBinaryType()     



@pytest.mark.xfail
def test_GetCommMask():
    GetCommMask()     



@pytest.mark.xfail
def test_GetCommModemStatus():
    GetCommModemStatus()     



@pytest.mark.xfail
def test_GetCommState():
    GetCommState()     



@pytest.mark.xfail
def test_GetCommTimeouts():
    GetCommTimeouts()     



@pytest.mark.xfail
def test_GetCompressedFileSize():
    GetCompressedFileSize()     



@pytest.mark.xfail
def test_GetDiskFreeSpace():
    GetDiskFreeSpace()     



@pytest.mark.xfail
def test_GetDiskFreeSpaceEx():
    GetDiskFreeSpaceEx()     



@pytest.mark.xfail
def test_GetDriveType():
    GetDriveType()     



@pytest.mark.xfail
def test_GetDriveTypeW():
    GetDriveTypeW()     



@pytest.mark.xfail
def test_GetFileAttributes():
    GetFileAttributes()     



@pytest.mark.xfail
def test_GetFileAttributesEx():
    GetFileAttributesEx()     



@pytest.mark.xfail
def test_GetFileAttributesW():
    GetFileAttributesW()     



@pytest.mark.xfail
def test_GetFileInformationByHandle():
    GetFileInformationByHandle()     



@pytest.mark.xfail
def test_GetFileSize():
    GetFileSize()     



@pytest.mark.xfail
def test_GetFileTime():
    GetFileTime()     



@pytest.mark.xfail
def test_GetFileType():
    GetFileType()     



@pytest.mark.xfail
def test_GetFinalPathNameByHandle():
    GetFinalPathNameByHandle()     



@pytest.mark.xfail
def test_GetFullPathName():
    GetFullPathName()     



def test_GetLogicalDrives():
    GetLogicalDrives()     



@pytest.mark.xfail
def test_GetLongPathName():
    GetLongPathName()     



@pytest.mark.xfail
def test_GetMailslotInfo():
    GetMailslotInfo()     



@pytest.mark.xfail
def test_GetOverlappedResult():
    GetOverlappedResult()     



@pytest.mark.xfail
def test_GetQueuedCompletionStatus():
    GetQueuedCompletionStatus()     



@pytest.mark.xfail
def test_GetVolumeNameForVolumeMountPoint():
    GetVolumeNameForVolumeMountPoint()     



@pytest.mark.xfail
def test_GetVolumePathName():
    GetVolumePathName()     



@pytest.mark.xfail
def test_GetVolumePathNamesForVolumeName():
    GetVolumePathNamesForVolumeName()     



@pytest.mark.xfail
def test_LockFile():
    LockFile()     



@pytest.mark.xfail
def test_LockFileEx():
    LockFileEx()     



@pytest.mark.xfail
def test_MoveFile():
    MoveFile()     



@pytest.mark.xfail
def test_MoveFileEx():
    MoveFileEx()     



@pytest.mark.xfail
def test_MoveFileExW():
    MoveFileExW()     



@pytest.mark.xfail
def test_MoveFileW():
    MoveFileW()     



@pytest.mark.xfail
def test_MoveFileWithProgress():
    MoveFileWithProgress()     



@pytest.mark.xfail
def test_OpenEncryptedFileRaw():
    OpenEncryptedFileRaw()     



@pytest.mark.xfail
def test_PostQueuedCompletionStatus():
    PostQueuedCompletionStatus()     



@pytest.mark.xfail
def test_PurgeComm():
    PurgeComm()     



@pytest.mark.xfail
def test_QueryDosDevice():
    QueryDosDevice()     



@pytest.mark.xfail
def test_QueryRecoveryAgentsOnEncryptedFile():
    QueryRecoveryAgentsOnEncryptedFile()     



@pytest.mark.xfail
def test_QueryUsersOnEncryptedFile():
    QueryUsersOnEncryptedFile()     



@pytest.mark.xfail
def test_ReadDirectoryChangesW():
    ReadDirectoryChangesW()     



@pytest.mark.xfail
def test_ReadEncryptedFileRaw():
    ReadEncryptedFileRaw()     



@pytest.mark.xfail
def test_ReadFile():
    ReadFile()     



@pytest.mark.xfail
def test_RemoveDirectory():
    RemoveDirectory()     



@pytest.mark.xfail
def test_RemoveUsersFromEncryptedFile():
    RemoveUsersFromEncryptedFile()     



@pytest.mark.xfail
def test_ReplaceFile():
    ReplaceFile()     



@pytest.mark.xfail
def test_SetCommBreak():
    SetCommBreak()     



@pytest.mark.xfail
def test_SetCommMask():
    SetCommMask()     



@pytest.mark.xfail
def test_SetCommState():
    SetCommState()     



@pytest.mark.xfail
def test_SetCommTimeouts():
    SetCommTimeouts()     



@pytest.mark.xfail
def test_SetCurrentDirectory():
    SetCurrentDirectory()     



@pytest.mark.xfail
def test_SetEndOfFile():
    SetEndOfFile()     



def test_SetFileApisToANSI():
    SetFileApisToANSI()     



def test_SetFileApisToOEM():
    SetFileApisToOEM()     



@pytest.mark.xfail
def test_SetFileAttributes():
    SetFileAttributes()     



@pytest.mark.xfail
def test_SetFileAttributesW():
    SetFileAttributesW()     



@pytest.mark.xfail
def test_SetFilePointer():
    SetFilePointer()     



@pytest.mark.xfail
def test_SetFileShortName():
    SetFileShortName()     



@pytest.mark.xfail
def test_SetFileTime():
    SetFileTime()     



@pytest.mark.xfail
def test_SetMailslotInfo():
    SetMailslotInfo()     



@pytest.mark.xfail
def test_SetVolumeLabel():
    SetVolumeLabel()     



@pytest.mark.xfail
def test_SetVolumeMountPoint():
    SetVolumeMountPoint()     



@pytest.mark.xfail
def test_SetupComm():
    SetupComm()     



def test_SfcGetNextProtectedFile():
    SfcGetNextProtectedFile()     



@pytest.mark.xfail
def test_SfcIsFileProtected():
    SfcIsFileProtected()     



@pytest.mark.xfail
def test_TransmitCommChar():
    TransmitCommChar()     



@pytest.mark.xfail
def test_TransmitFile():
    TransmitFile()     



@pytest.mark.xfail
def test_UnlockFile():
    UnlockFile()     



@pytest.mark.xfail
def test_UnlockFileEx():
    UnlockFileEx()     



@pytest.mark.xfail
def test_WSAAsyncSelect():
    WSAAsyncSelect()     



@pytest.mark.xfail
def test_WSAEventSelect():
    WSAEventSelect()     



@pytest.mark.xfail
def test_WSARecv():
    WSARecv()     



@pytest.mark.xfail
def test_WSASend():
    WSASend()     



@pytest.mark.xfail
def test_WaitCommEvent():
    WaitCommEvent()     



@pytest.mark.xfail
def test_WriteEncryptedFileRaw():
    WriteEncryptedFileRaw()     



@pytest.mark.xfail
def test_WriteFile():
    WriteFile()     



@pytest.mark.xfail
def test_get_osfhandle():
    get_osfhandle()     



@pytest.mark.xfail
def test_getmaxstdio():
    getmaxstdio()     



@pytest.mark.xfail
def test_open_osfhandle():
    open_osfhandle()     



@pytest.mark.xfail
def test_setmaxstdio():
    setmaxstdio()     





import pytest
import pytest
from win32help import *
def test_HHNTRACK():
    HHNTRACK()     



def test_HHN_NOTIFY():
    HHN_NOTIFY()     



def test_HH_AKLINK():
    HH_AKLINK()     



def test_HH_FTS_QUERY():
    HH_FTS_QUERY()     



def test_HH_POPUP():
    HH_POPUP()     



def test_HH_WINTYPE():
    HH_WINTYPE()     



@pytest.mark.xfail
def test_HtmlHelp():
    HtmlHelp()     



def test_NMHDR():
    NMHDR()     



@pytest.mark.xfail
def test_WinHelp():
    WinHelp()     





from exchdapi import *
def test_HrAdminProgramExists():
    HrAdminProgramExists()     



def test_HrCreateGatewayProfile():
    HrCreateGatewayProfile()     



def test_HrCreateMailboxAgentProfile():
    HrCreateMailboxAgentProfile()     



def test_HrEnumContainers():
    HrEnumContainers()     



def test_HrEnumOrganizations():
    HrEnumOrganizations()     



def test_HrEnumSiteAdmins():
    HrEnumSiteAdmins()     



def test_HrEnumSites():
    HrEnumSites()     



def test_HrGetServiceAccountName():
    HrGetServiceAccountName()     



def test_HrInstallMailboxAgent():
    HrInstallMailboxAgent()     



def test_HrInstallService():
    HrInstallService()     



def test_HrMailboxAgentExists():
    HrMailboxAgentExists()     



def test_HrRemoveMailboxAgent():
    HrRemoveMailboxAgent()     



def test_HrRemoveProfile():
    HrRemoveProfile()     





import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from servicemanager import *
@pytest.mark.xfail
def test_CoInitializeEx():
    CoInitializeEx()     



def test_CoUninitialize():
    CoUninitialize()     



def test_Debugging():
    Debugging()     



def test_Finalize():
    Finalize()     



def test_Initialize():
    Initialize()     



@pytest.mark.xfail
def test_LogErrorMsg():
    LogErrorMsg()     



@pytest.mark.xfail
def test_LogInfoMsg():
    LogInfoMsg()     



@pytest.mark.xfail
def test_LogMsg():
    LogMsg()     



@pytest.mark.xfail
def test_LogWarningMsg():
    LogWarningMsg()     



@pytest.mark.xfail
def test_PrepareToHostMultiple():
    PrepareToHostMultiple()     



def test_PrepareToHostSingle():
    PrepareToHostSingle()     



def test_PumpWaitingMessages():
    PumpWaitingMessages()     



@pytest.mark.xfail
def test_RegisterServiceCtrlHandler():
    RegisterServiceCtrlHandler()     



def test_RunningAsService():
    RunningAsService()     



@pytest.mark.xfail
def test_SetEventSourceName():
    SetEventSourceName()     



@pytest.mark.xfail
def test_StartServiceCtrlDispatcher():
    StartServiceCtrlDispatcher()     





import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32console import *
@pytest.mark.xfail
def test_AddConsoleAlias():
    AddConsoleAlias()     



@pytest.mark.xfail
def test_AllocConsole():
    AllocConsole()     



@pytest.mark.xfail
def test_AttachConsole():
    AttachConsole()     



def test_CreateConsoleScreenBuffer():
    CreateConsoleScreenBuffer()     



def test_FreeConsole():
    FreeConsole()     



@pytest.mark.xfail
def test_GenerateConsoleCtrlEvent():
    GenerateConsoleCtrlEvent()     



def test_GetConsoleAliasExes():
    GetConsoleAliasExes()     



@pytest.mark.xfail
def test_GetConsoleAliases():
    GetConsoleAliases()     



def test_GetConsoleCP():
    GetConsoleCP()     



@pytest.mark.xfail
def test_GetConsoleDisplayMode():
    GetConsoleDisplayMode()     



def test_GetConsoleOutputCP():
    GetConsoleOutputCP()     



@pytest.mark.xfail
def test_GetConsoleProcessList():
    GetConsoleProcessList()     



@pytest.mark.xfail
@pytest.mark.xfail
def test_GetConsoleSelectionInfo():
    GetConsoleSelectionInfo()     



@pytest.mark.xfail
def test_GetConsoleTitle():
    GetConsoleTitle()     



def test_GetConsoleWindow():
    GetConsoleWindow()     



def test_GetNumberOfConsoleFonts():
    GetNumberOfConsoleFonts()     



@pytest.mark.xfail
def test_GetStdHandle():
    GetStdHandle()     



@pytest.mark.xfail
def test_SetConsoleCP():
    SetConsoleCP()     



@pytest.mark.xfail
def test_SetConsoleOutputCP():
    SetConsoleOutputCP()     



@pytest.mark.xfail
def test_SetConsoleTitle():
    SetConsoleTitle()     





import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32transaction import *
@pytest.mark.xfail
def test_CommitTransaction():
    CommitTransaction()     



@pytest.mark.xfail
def test_CommitTransactionAsync():
    CommitTransactionAsync()     



def test_CreateTransaction():
    CreateTransaction()     



@pytest.mark.xfail
def test_GetTransactionId():
    GetTransactionId()     



@pytest.mark.xfail
def test_OpenTransaction():
    OpenTransaction()     



@pytest.mark.xfail
def test_RollbackTransaction():
    RollbackTransaction()     



@pytest.mark.xfail
def test_RollbackTransactionAsync():
    RollbackTransactionAsync()     





# odbc test suite kindly contributed by Frank Millman.
import odbc
import os
import sys
import tempfile
import unittest

import pytest
import pythoncom
from pywin32_testutil import str2bytes, str2memory, TestSkipped
from win32com.client import constants
from win32com.client.gencache import EnsureDispatch


class TestStuff(unittest.TestCase):
    def setUp(self):
        self.tablename = "pywin32test_users"
        self.db_filename = None
        self.conn = self.cur = None
        try:
            # Test any database if a connection string is supplied...
            conn_str = os.environ['TEST_ODBC_CONNECTION_STRING']
        except KeyError:
            # Create a local MSAccess DB for testing.
            self.db_filename = tempfile.NamedTemporaryFile().name + '.mdb'

            # Create a brand-new database - what is the story with these?
            for suffix in (".36", ".35", ".30"):
                try:
                    dbe = EnsureDispatch("DAO.DBEngine" + suffix)
                    break
                except pythoncom.com_error:
                    pass
            else:
                raise TestSkipped("Can't find a DB engine")

            workspace = dbe.Workspaces(0)

            newdb = workspace.CreateDatabase(self.db_filename,
                                             constants.dbLangGeneral,
                                             constants.dbEncrypt)

            newdb.Close()

            conn_str = "Driver={Microsoft Access Driver (*.mdb)};dbq=%s;Uid=;Pwd=;" \
                       % (self.db_filename,)
        # print 'Connection string:', conn_str
        self.conn = odbc.odbc(conn_str)
        # And we expect a 'users' table for these tests.
        self.cur = self.conn.cursor()
        # self.cur.setoutputsize(1000)
        try:
            self.cur.execute("""drop table %s""" % self.tablename)
        except (odbc.error, odbc.progError):
            pass

        # This needs to be adjusted for sql server syntax for unicode fields
        # - memo -> TEXT
        # - varchar -> nvarchar
        assert self.cur.execute(
            """create table %s (
                    userid varchar(25),
                    username varchar(25),
                    bitfield bit,
                    intfield integer,
                    floatfield float,
                    datefield datetime,
                    rawfield varbinary(100),
                    longtextfield memo,
                    longbinaryfield image
            )""" % self.tablename) == -1

    def tearDown(self):
        if self.cur is not None:
            try:
                self.cur.execute("""drop table %s""" % self.tablename)
            except (odbc.error, odbc.progError) as why:
                print("Failed to delete test table %s" % self.tablename, why)

            self.cur.close()
            self.cur = None
        if self.conn is not None:
            self.conn.close()
            self.conn = None
        if self.db_filename is not None:
            try:
                os.unlink(self.db_filename)
            except OSError:
                pass

    @pytest.mark.xfail
    def test_insert_select(self, userid='Frank', username='Frank Millman'):
        assert self.cur.execute("insert into %s (userid, username) \
            values (?,?)" % self.tablename, [userid, username]) == 1
        assert self.cur.execute("select * from %s \
            where userid = ?" % self.tablename, [userid.lower()]) == 0
        assert self.cur.execute("select * from %s \
            where username = ?" % self.tablename, [username.lower()]) == 0

    @pytest.mark.xfail
    def test_insert_select_unicode(
            self, userid='Frank', username="Frank Millman"):
        assert self.cur.execute("insert into %s (userid, username)\
            values (?,?)" % self.tablename, [userid, username]) == 1
        assert self.cur.execute("select * from %s \
            where userid = ?" % self.tablename, [userid.lower()]) == 0
        assert self.cur.execute("select * from %s \
            where username = ?" % self.tablename, [username.lower()]) == 0

    @pytest.mark.xfail
    def test_insert_select_unicode_ext(self):
        userid = "t-\xe0\xf2"
        username = "test-\xe0\xf2 name"
        self.test_insert_select_unicode(userid, username)

    def _test_val(self, fieldName, value):
        for x in range(100):
            self.cur.execute(
                "delete from %s where userid='Frank'" %
                self.tablename)
            assert self.cur.execute(
                "insert into %s (userid, %s) values (?,?)" % (
                    self.tablename, fieldName),
                ["Frank", value]) == 1
            self.cur.execute(
                "select %s from %s where userid = ?" %
                (fieldName, self.tablename), ["Frank"])
            rows = self.cur.fetchmany()
            assert 1 == len(rows)
            row = rows[0]
            assert row[0] == value

    @pytest.mark.xfail
    def testBit(self):
        self._test_val('bitfield', 1)
        self._test_val('bitfield', 0)

    @pytest.mark.xfail
    def testInt(self):
        self._test_val('intfield', 1)
        self._test_val('intfield', 0)
        try:
            big = sys.maxsize
        except AttributeError:
            big = sys.maxsize
        self._test_val('intfield', big)

    @pytest.mark.xfail
    def testFloat(self):
        self._test_val('floatfield', 1.01)
        self._test_val('floatfield', 0)

    @pytest.mark.xfail
    def testVarchar(self, ):
        self._test_val('username', 'foo')

    @pytest.mark.xfail
    def testLongVarchar(self):
        """ Test a long text field in excess of internal cursor data size (65536)"""
        self._test_val('longtextfield', 'abc' * 70000)

    @pytest.mark.xfail
    def testLongBinary(self):
        """ Test a long raw field in excess of internal cursor data size (65536)"""
        self._test_val('longbinaryfield', str2memory('\0\1\2' * 70000))

    @pytest.mark.xfail
    def testRaw(self):
        # Test binary data
        self._test_val('rawfield', str2memory('\1\2\3\4\0\5\6\7\8'))

    @pytest.mark.xfail
    def test_widechar(self):
        """Test a unicode character that would be mangled if bound as plain character.
            For example, previously the below was returned as ascii 'a'
        """
        self._test_val('username', '\u0101')

    @pytest.mark.xfail
    def testDates(self):
        import datetime
        for v in (
                (1900, 12, 25, 23, 39, 59),
        ):
            d = datetime.datetime(*v)
            self._test_val('datefield', d)

    @pytest.mark.xfail
    def test_set_nonzero_length(self):
        assert self.cur.execute(
                "insert into %s (userid,username) "
                "values (?,?)" %
                self.tablename, [
                'Frank', 'Frank Millman']) == 1
        assert self.cur.execute(
                "update %s set username = ?" %
                self.tablename, ['Frank']) == 1
        assert self.cur.execute(
                "select * from %s" %
                self.tablename) == 0
        assert len(self.cur.fetchone()[1]) == 5

    @pytest.mark.xfail
    def test_set_zero_length(self):
        assert self.cur.execute(
                "insert into %s (userid,username) "
                "values (?,?)" %
                self.tablename, [
                str2bytes('Frank'), '']) == 1
        assert self.cur.execute(
                "select * from %s" %
                self.tablename) == 0
        assert len(self.cur.fetchone()[1]) == 0

    @pytest.mark.xfail
    def test_set_zero_length_unicode(self):
        assert self.cur.execute(
                "insert into %s (userid,username) "
                "values (?,?)" %
                self.tablename, [
                'Frank', '']) == 1
        assert self.cur.execute(
                "select * from %s" %
                self.tablename) == 0
        assert len(self.cur.fetchone()[1]) == 0


if __name__ == '__main__':
    unittest.main()



import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32service import *
@pytest.mark.xfail
def test_ChangeServiceConfig():
    ChangeServiceConfig()     



@pytest.mark.xfail
def test_ChangeServiceConfig2():
    ChangeServiceConfig2()     



@pytest.mark.xfail
def test_CloseServiceHandle():
    CloseServiceHandle()     



@pytest.mark.xfail
def test_ControlService():
    ControlService()     



@pytest.mark.xfail
def test_CreateDesktop():
    CreateDesktop()     



@pytest.mark.xfail
def test_CreateService():
    CreateService()     



@pytest.mark.xfail
def test_CreateWindowStation():
    CreateWindowStation()     



@pytest.mark.xfail
def test_DeleteService():
    DeleteService()     



@pytest.mark.xfail
def test_EnumDependentServices():
    EnumDependentServices()     



@pytest.mark.xfail
def test_EnumServicesStatus():
    EnumServicesStatus()     



def test_EnumWindowStations():
    EnumWindowStations()     



def test_GetProcessWindowStation():
    GetProcessWindowStation()     



@pytest.mark.xfail
def test_GetServiceDisplayName():
    GetServiceDisplayName()     



@pytest.mark.xfail
def test_GetServiceKeyName():
    GetServiceKeyName()     



@pytest.mark.xfail
def test_GetThreadDesktop():
    GetThreadDesktop()     



@pytest.mark.xfail
def test_GetUserObjectInformation():
    GetUserObjectInformation()     



@pytest.mark.xfail
def test_LockServiceDatabase():
    LockServiceDatabase()     



@pytest.mark.xfail
def test_OpenDesktop():
    OpenDesktop()     



@pytest.mark.xfail
def test_OpenInputDesktop():
    OpenInputDesktop()     



@pytest.mark.xfail
def test_OpenSCManager():
    OpenSCManager()     



@pytest.mark.xfail
def test_OpenService():
    OpenService()     



@pytest.mark.xfail
def test_OpenWindowStation():
    OpenWindowStation()     



@pytest.mark.xfail
def test_QueryServiceConfig():
    QueryServiceConfig()     



@pytest.mark.xfail
def test_QueryServiceConfig2():
    QueryServiceConfig2()     



@pytest.mark.xfail
def test_QueryServiceLockStatus():
    QueryServiceLockStatus()     



@pytest.mark.xfail
def test_QueryServiceObjectSecurity():
    QueryServiceObjectSecurity()     



@pytest.mark.xfail
def test_QueryServiceStatus():
    QueryServiceStatus()     



@pytest.mark.xfail
def test_QueryServiceStatusEx():
    QueryServiceStatusEx()     



@pytest.mark.xfail
def test_SetServiceObjectSecurity():
    SetServiceObjectSecurity()     



@pytest.mark.xfail
def test_SetServiceStatus():
    SetServiceStatus()     



@pytest.mark.xfail
def test_SetUserObjectInformation():
    SetUserObjectInformation()     



@pytest.mark.xfail
def test_StartService():
    StartService()     



@pytest.mark.xfail
def test_UnlockServiceDatabase():
    UnlockServiceDatabase()     





import pytest
import pytest
import pytest
import pytest
import pytest

from pywintypes import *


def test_ACL():
    ACL()


def test_CreateGuid():
    CreateGuid()


@pytest.mark.xfail
def test_DosDateTimeToTime():
    DosDateTimeToTime()


def test_HANDLE():
    HANDLE()


def test_HKEY():
    HKEY()


@pytest.mark.xfail
def test_IID():
    IID()


@pytest.mark.xfail
def test_IsTextUnicode():
    IsTextUnicode()


def test_OVERLAPPED():
    OVERLAPPED()


def test_SECURITY_ATTRIBUTES():
    SECURITY_ATTRIBUTES()


def test_SECURITY_DESCRIPTOR():
    SECURITY_DESCRIPTOR()


def test_SID():
    SID()


@pytest.mark.xfail
def test_Time():
    Time()


@pytest.mark.skip
def test_Unicode():
    Unicode()


@pytest.mark.skip
def test_UnicodeFromRaw():
    UnicodeFromRaw()


def test_WAVEFORMATEX():
    WAVEFORMATEX()



'''This is a distutils setup-script for the pywin32 extensions

To build the pywin32 extensions, simply execute:
  python setup.py build
or
  python setup.py install
to build and install into your current Python installation.
'''

from setuptools import setup, find_packages, Distribution

# Install all required packages before running anything else
Distribution({
    'setup_requires': [
        'pyyaml',
        'toposort',
        'setuptools_scm',
    ]
})

import os
import re
import textwrap

from win32.distutils.command import win32_build_ext
from win32.distutils.util import collect_extensions
from distutils.sysconfig import get_config_vars
from contextlib import suppress

# prevent the new in 3.5 suffix of 'cpXX-win32' from being added.
# (adjusting both .cp35-win_amd64.pyd and .cp35-win32.pyd to .pyd)
with suppress(KeyError):
    get_config_vars()["EXT_SUFFIX"] = re.sub("\\.cp\d\d-win((32)|(_amd64))",
                                             "",
                                             get_config_vars()["EXT_SUFFIX"])

setup(
    name='win32core',
    description='Python for Window Extensions',
    long_description=textwrap.dedent('''
        Python extensions for Microsoft Windows'
        Provides access to much of the Win32 API, the
        ability to create and use COM objects, and the
        Pythonwin environment
        
        This provides the core functionality.
        '''),
    author='Mark Hammond (et al)',
    author_email='mhammond@users.sourceforge.net',
    url='http://sourceforge.net/projects/pywin32/',
    license='PSF',
    cmdclass={
        'build_ext': win32_build_ext,
    },
    ext_modules=list(collect_extensions()),
    packages=find_packages(),
    use_scm_version=True,
    entry_points={
        'console_scripts': [
            'makepy = win32.com.client.makepy:main',
            'verstamp = win32.verstamp:main',
        ],
    })



# General purpose service utilities, both for standard Python scripts,
# and for for Python programs which run as services...
#
# Note that most utility functions here will raise win32api.error's
# (which is == win32service.error, pywintypes.error, etc)
# when things go wrong - eg, not enough permissions to hit the
# registry etc.

import os
import sys
import warnings

import win32types as pywintypes
from win32 import api as win32api
from win32 import constants as win32con
from win32 import service as win32service
from win32 import error as win32error

error = RuntimeError


def LocatePythonServiceExe(exeName=None):
    if not exeName and hasattr(sys, "frozen"):
        # If py2exe etc calls this with no exeName, default is current exe.
        return sys.executable

    # Try and find the specified EXE somewhere.  If specifically registered,
    # use it.  Otherwise look down sys.path, and the global PATH environment.
    if exeName is None:
        if os.path.splitext(win32service.__file__)[0].endswith("_d"):
            exeName = "PythonService_d.exe"
        else:
            exeName = "PythonService.exe"
    # See if it exists as specified
    if os.path.isfile(exeName):
        return win32api.GetFullPathName(exeName)
    baseName = os.path.splitext(os.path.basename(exeName))[0]
    try:
        exeName = win32api.RegQueryValue(win32con.HKEY_LOCAL_MACHINE,
                                         "Software\\Python\\%s\\%s" %
                                         (baseName, sys.winver))
        if os.path.isfile(exeName):
            return exeName
        raise RuntimeError(
            "The executable '%s' is registered as the Python "
            "service exe, but it does not exist as specified" % exeName)
    except win32api.error:
        # OK - not there - lets go a-searchin'
        for path in [sys.prefix] + sys.path:
            look = os.path.join(path, exeName)
            if os.path.isfile(look):
                return win32api.GetFullPathName(look)
        # Try the global Path.
        try:
            return win32api.SearchPath(None, exeName)[0]
        except win32api.error:
            msg = "%s is not correctly registered\nPlease locate and run %s, and it will self-register\nThen run this service registration process again." % (
                exeName, exeName)
            raise error(msg)


def _GetServiceShortName(longName):
    # looks up a services name
    # from the display name
    # Thanks to Andy McKay for this code.
    access = win32con.KEY_READ | win32con.KEY_ENUMERATE_SUB_KEYS | win32con.KEY_QUERY_VALUE
    hkey = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE,
                               "SYSTEM\\CurrentControlSet\\Services", 0,
                               access)
    num = win32api.RegQueryInfoKey(hkey)[0]
    longName = longName.lower()
    # loop through number of subkeys
    for x in range(0, num):
        # find service name, open subkey
        svc = win32api.RegEnumKey(hkey, x)
        skey = win32api.RegOpenKey(hkey, svc, 0, access)
        try:
            # find display name
            thisName = str(win32api.RegQueryValueEx(skey, "DisplayName")[0])
            if thisName.lower() == longName:
                return svc
        except win32api.error:
            # in case there is no key called DisplayName
            pass
    return None


# Open a service given either it's long or short name.


def SmartOpenService(hscm, name, access):
    try:
        return win32service.OpenService(hscm, name, access)
    except win32api.error as details:
        if details.winerror not in [
                winerror.ERROR_SERVICE_DOES_NOT_EXIST,
                winerror.ERROR_INVALID_NAME
        ]:
            raise
    name = win32service.GetServiceKeyName(hscm, name)
    return win32service.OpenService(hscm, name, access)


def LocateSpecificServiceExe(serviceName):
    # Given the name of a specific service, return the .EXE name _it_ uses
    # (which may or may not be the Python Service EXE
    hkey = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE,
                               "SYSTEM\\CurrentControlSet\\Services\\%s" %
                               (serviceName), 0, win32con.KEY_ALL_ACCESS)
    try:
        return win32api.RegQueryValueEx(hkey, "ImagePath")[0]
    finally:
        hkey.Close()


def InstallPerfmonForService(serviceName, iniName, dllName=None):
    # If no DLL name, look it up in the INI file name
    if not dllName:  # May be empty string!
        dllName = win32api.GetProfileVal("Python", "dll", "", iniName)
    # Still not found - look for the standard one in the same dir as
    # win32service.pyd
    if not dllName:
        try:
            tryName = os.path.join(
                os.path.split(win32service.__file__)[0], "perfmondata.dll")
            if os.path.isfile(tryName):
                dllName = tryName
        except AttributeError:
            # Frozen app? - anyway, can't find it!
            pass
    if not dllName:
        raise ValueError("The name of the performance DLL must be available")
    dllName = win32api.GetFullPathName(dllName)
    # Now setup all the required "Performance" entries.
    hkey = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE,
                               "SYSTEM\\CurrentControlSet\\Services\\%s" %
                               (serviceName), 0, win32con.KEY_ALL_ACCESS)
    try:
        subKey = win32api.RegCreateKey(hkey, "Performance")
        try:
            win32api.RegSetValueEx(subKey, "Library", 0, win32con.REG_SZ,
                                   dllName)
            win32api.RegSetValueEx(subKey, "Open", 0, win32con.REG_SZ,
                                   "OpenPerformanceData")
            win32api.RegSetValueEx(subKey, "Close", 0, win32con.REG_SZ,
                                   "ClosePerformanceData")
            win32api.RegSetValueEx(subKey, "Collect", 0, win32con.REG_SZ,
                                   "CollectPerformanceData")
        finally:
            win32api.RegCloseKey(subKey)
    finally:
        win32api.RegCloseKey(hkey)
    # Now do the "Lodctr" thang...

    try:
        import perfmon
        path, fname = os.path.split(iniName)
        oldPath = os.getcwd()
        if path:
            os.chdir(path)
        try:
            perfmon.LoadPerfCounterTextStrings("python.exe " + fname)
        finally:
            os.chdir(oldPath)
    except win32api.error as details:
        print("The service was installed OK, but the performance monitor")
        print("data could not be loaded.", details)


def _GetCommandLine(exeName, exeArgs):
    if exeArgs is not None:
        return exeName + " " + exeArgs
    else:
        return exeName


def InstallService(pythonClassString,
                   serviceName,
                   displayName,
                   startType=None,
                   errorControl=None,
                   bRunInteractive=0,
                   serviceDeps=None,
                   userName=None,
                   password=None,
                   exeName=None,
                   perfMonIni=None,
                   perfMonDll=None,
                   exeArgs=None,
                   description=None,
                   delayedstart=None):
    # Handle the default arguments.
    if startType is None:
        startType = win32service.SERVICE_DEMAND_START
    serviceType = win32service.SERVICE_WIN32_OWN_PROCESS
    if bRunInteractive:
        serviceType = serviceType | win32service.SERVICE_INTERACTIVE_PROCESS
    if errorControl is None:
        errorControl = win32service.SERVICE_ERROR_NORMAL

    # None here means use default PythonService.exe
    exeName = '"%s"' % LocatePythonServiceExe(exeName)
    commandLine = _GetCommandLine(exeName, exeArgs)
    hscm = win32service.OpenSCManager(None, None,
                                      win32service.SC_MANAGER_ALL_ACCESS)
    try:
        hs = win32service.CreateService(
            hscm,
            serviceName,
            displayName,
            win32service.SERVICE_ALL_ACCESS,  # desired access
            serviceType,  # service type
            startType,
            errorControl,  # error control type
            commandLine,
            None,
            0,
            serviceDeps,
            userName,
            password)
        if description is not None:
            try:
                win32service.ChangeServiceConfig2(
                    hs, win32service.SERVICE_CONFIG_DESCRIPTION, description)
            except NotImplementedError:
                pass  # ChangeServiceConfig2 and description do not exist on NT
        if delayedstart is not None:
            try:
                win32service.ChangeServiceConfig2(
                    hs, win32service.SERVICE_CONFIG_DELAYED_AUTO_START_INFO,
                    delayedstart)
            except (win32service.error, NotImplementedError):
                # delayed start only exists on Vista and later - warn only when
                # trying to set delayed to True
                if delayedstart:
                    warnings.warn('Delayed Start not available on this system')
        win32service.CloseServiceHandle(hs)
    finally:
        win32service.CloseServiceHandle(hscm)
    InstallPythonClassString(pythonClassString, serviceName)
    # If I have performance monitor info to install, do that.
    if perfMonIni is not None:
        InstallPerfmonForService(serviceName, perfMonIni, perfMonDll)


def ChangeServiceConfig(pythonClassString,
                        serviceName,
                        startType=None,
                        errorControl=None,
                        bRunInteractive=0,
                        serviceDeps=None,
                        userName=None,
                        password=None,
                        exeName=None,
                        displayName=None,
                        perfMonIni=None,
                        perfMonDll=None,
                        exeArgs=None,
                        description=None,
                        delayedstart=None):
    # Before doing anything, remove any perfmon counters.
    try:
        import perfmon
        perfmon.UnloadPerfCounterTextStrings("python.exe " + serviceName)
    except (ImportError, win32api.error):
        pass

    # The EXE location may have changed
    exeName = '"%s"' % LocatePythonServiceExe(exeName)

    # Handle the default arguments.
    if startType is None:
        startType = win32service.SERVICE_NO_CHANGE
    if errorControl is None:
        errorControl = win32service.SERVICE_NO_CHANGE

    hscm = win32service.OpenSCManager(None, None,
                                      win32service.SC_MANAGER_ALL_ACCESS)
    serviceType = win32service.SERVICE_WIN32_OWN_PROCESS
    if bRunInteractive:
        serviceType = serviceType | win32service.SERVICE_INTERACTIVE_PROCESS
    commandLine = _GetCommandLine(exeName, exeArgs)
    try:
        hs = SmartOpenService(hscm, serviceName,
                              win32service.SERVICE_ALL_ACCESS)
        try:

            win32service.ChangeServiceConfig(
                hs,
                serviceType,  # service type
                startType,
                errorControl,  # error control type
                commandLine,
                None,
                0,
                serviceDeps,
                userName,
                password,
                displayName)
            if description is not None:
                try:
                    win32service.ChangeServiceConfig2(
                        hs, win32service.SERVICE_CONFIG_DESCRIPTION,
                        description)
                except NotImplementedError:
                    pass  # ChangeServiceConfig2 and description do not exist on NT
            if delayedstart is not None:
                try:
                    win32service.ChangeServiceConfig2(
                        hs,
                        win32service.SERVICE_CONFIG_DELAYED_AUTO_START_INFO,
                        delayedstart)
                except (win32service.error, NotImplementedError):
                    # Delayed start only exists on Vista and later.  On Nt, will raise NotImplementedError since ChangeServiceConfig2
                    # doensn't exist.  On Win2k and XP, will fail with ERROR_INVALID_LEVEL
                    # Warn only if trying to set delayed to True
                    if delayedstart:
                        warnings.warn(
                            'Delayed Start not available on this system')
        finally:
            win32service.CloseServiceHandle(hs)
    finally:
        win32service.CloseServiceHandle(hscm)
    InstallPythonClassString(pythonClassString, serviceName)
    # If I have performance monitor info to install, do that.
    if perfMonIni is not None:
        InstallPerfmonForService(serviceName, perfMonIni, perfMonDll)


def InstallPythonClassString(pythonClassString, serviceName):
    # Now setup our Python specific entries.
    if pythonClassString:
        key = win32api.RegCreateKey(
            win32con.HKEY_LOCAL_MACHINE,
            "System\\CurrentControlSet\\Services\\%s\\PythonClass" %
            serviceName)
        try:
            win32api.RegSetValue(key, None, win32con.REG_SZ, pythonClassString)
        finally:
            win32api.RegCloseKey(key)


# Utility functions for Services, to allow persistant properties.


def SetServiceCustomOption(serviceName, option, value):
    try:
        serviceName = serviceName._svc_name_
    except AttributeError:
        pass
    key = win32api.RegCreateKey(
        win32con.HKEY_LOCAL_MACHINE,
        "System\\CurrentControlSet\\Services\\%s\\Parameters" % serviceName)
    try:
        if isinstance(value, type(0)):
            win32api.RegSetValueEx(key, option, 0, win32con.REG_DWORD, value)
        else:
            win32api.RegSetValueEx(key, option, 0, win32con.REG_SZ, value)
    finally:
        win32api.RegCloseKey(key)


def GetServiceCustomOption(serviceName, option, defaultValue=None):
    # First param may also be a service class/instance.
    # This allows services to pass "self"
    try:
        serviceName = serviceName._svc_name_
    except AttributeError:
        pass
    key = win32api.RegCreateKey(
        win32con.HKEY_LOCAL_MACHINE,
        "System\\CurrentControlSet\\Services\\%s\\Parameters" % serviceName)
    try:
        try:
            return win32api.RegQueryValueEx(key, option)[0]
        except win32api.error:  # No value.
            return defaultValue
    finally:
        win32api.RegCloseKey(key)


def RemoveService(serviceName):
    try:
        import perfmon
        perfmon.UnloadPerfCounterTextStrings("python.exe " + serviceName)
    except (ImportError, win32api.error):
        pass

    hscm = win32service.OpenSCManager(None, None,
                                      win32service.SC_MANAGER_ALL_ACCESS)
    try:
        hs = SmartOpenService(hscm, serviceName,
                              win32service.SERVICE_ALL_ACCESS)
        win32service.DeleteService(hs)
        win32service.CloseServiceHandle(hs)
    finally:
        win32service.CloseServiceHandle(hscm)

    import win32evtlogutil
    try:
        win32evtlogutil.RemoveSourceFromRegistry(serviceName)
    except win32api.error:
        pass


def ControlService(serviceName, code, machine=None):
    hscm = win32service.OpenSCManager(machine, None,
                                      win32service.SC_MANAGER_ALL_ACCESS)
    try:

        hs = SmartOpenService(hscm, serviceName,
                              win32service.SERVICE_ALL_ACCESS)
        try:
            status = win32service.ControlService(hs, code)
        finally:
            win32service.CloseServiceHandle(hs)
    finally:
        win32service.CloseServiceHandle(hscm)
    return status


def __FindSvcDeps(findName):
    if isinstance(findName, pywintypes.UnicodeType):
        findName = str(findName)
    dict = {}
    k = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE,
                            "SYSTEM\\CurrentControlSet\\Services")
    num = 0
    while True:
        try:
            svc = win32api.RegEnumKey(k, num)
        except win32api.error:
            break
        num = num + 1
        sk = win32api.RegOpenKey(k, svc)
        try:
            deps, typ = win32api.RegQueryValueEx(sk, "DependOnService")
        except win32api.error:
            deps = ()
        for dep in deps:
            dep = dep.lower()
            dep_on = dict.get(dep, [])
            dep_on.append(svc)
            dict[dep] = dep_on

    return __ResolveDeps(findName, dict)


def __ResolveDeps(findName, dict):
    items = dict.get(findName.lower(), [])
    retList = []
    for svc in items:
        retList.insert(0, svc)
        retList = __ResolveDeps(svc, dict) + retList
    return retList


def WaitForServiceStatus(serviceName, status, waitSecs, machine=None):
    """Waits for the service to return the specified status.  You
    should have already requested the service to enter that state"""
    for i in range(waitSecs * 4):
        now_status = QueryServiceStatus(serviceName, machine)[1]
        if now_status == status:
            break
        win32api.Sleep(250)
    else:
        raise pywintypes.error(
            winerror.ERROR_SERVICE_REQUEST_TIMEOUT, "QueryServiceStatus",
            win32api.FormatMessage(
                winerror.ERROR_SERVICE_REQUEST_TIMEOUT)[:-2])


def __StopServiceWithTimeout(hs, waitSecs=30):
    try:
        status = win32service.ControlService(hs,
                                             win32service.SERVICE_CONTROL_STOP)
    except pywintypes.error as exc:
        if exc.winerror != winerror.ERROR_SERVICE_NOT_ACTIVE:
            raise
    for i in range(waitSecs):
        status = win32service.QueryServiceStatus(hs)
        if status[1] == win32service.SERVICE_STOPPED:
            break
        win32api.Sleep(1000)
    else:
        raise pywintypes.error(
            winerror.ERROR_SERVICE_REQUEST_TIMEOUT, "ControlService",
            win32api.FormatMessage(
                winerror.ERROR_SERVICE_REQUEST_TIMEOUT)[:-2])


def StopServiceWithDeps(serviceName, machine=None, waitSecs=30):
    # Stop a service recursively looking for dependant services
    hscm = win32service.OpenSCManager(machine, None,
                                      win32service.SC_MANAGER_ALL_ACCESS)
    try:
        deps = __FindSvcDeps(serviceName)
        for dep in deps:
            hs = win32service.OpenService(hscm, dep,
                                          win32service.SERVICE_ALL_ACCESS)
            try:
                __StopServiceWithTimeout(hs, waitSecs)
            finally:
                win32service.CloseServiceHandle(hs)
        # Now my service!
        hs = win32service.OpenService(hscm, serviceName,
                                      win32service.SERVICE_ALL_ACCESS)
        try:
            __StopServiceWithTimeout(hs, waitSecs)
        finally:
            win32service.CloseServiceHandle(hs)

    finally:
        win32service.CloseServiceHandle(hscm)


def StopService(serviceName, machine=None):
    return ControlService(serviceName, win32service.SERVICE_CONTROL_STOP,
                          machine)


def StartService(serviceName, args=None, machine=None):
    hscm = win32service.OpenSCManager(machine, None,
                                      win32service.SC_MANAGER_ALL_ACCESS)
    try:

        hs = SmartOpenService(hscm, serviceName,
                              win32service.SERVICE_ALL_ACCESS)
        try:
            win32service.StartService(hs, args)
        finally:
            win32service.CloseServiceHandle(hs)
    finally:
        win32service.CloseServiceHandle(hscm)


def RestartService(serviceName, args=None, waitSeconds=30, machine=None):
    "Stop the service, and then start it again (with some tolerance for allowing it to stop.)"
    try:
        StopService(serviceName, machine)
    except pywintypes.error as exc:
        # Allow only "service not running" error
        if exc.winerror != winerror.ERROR_SERVICE_NOT_ACTIVE:
            raise
    # Give it a few goes, as the service may take time to stop
    for i in range(waitSeconds):
        try:
            StartService(serviceName, args, machine)
            break
        except pywintypes.error as exc:
            if exc.winerror != winerror.ERROR_SERVICE_ALREADY_RUNNING:
                raise
            win32api.Sleep(1000)
    else:
        print("Gave up waiting for the old service to stop!")


def _DebugCtrlHandler(evt):
    if evt in (win32con.CTRL_C_EVENT, win32con.CTRL_BREAK_EVENT):
        assert g_debugService
        print("Stopping debug service.")
        g_debugService.SvcStop()
        return True
    return False


def DebugService(cls, argv=[]):
    # Run a service in "debug" mode.  Re-implements what pythonservice.exe
    # does when it sees a "-debug" param.
    # Currently only used by "frozen" (ie, py2exe) programs (but later may
    # end up being used for all services should we ever remove
    # pythonservice.exe)
    import servicemanager
    global g_debugService

    print("Debugging service %s - press Ctrl+C to stop." % (cls._svc_name_, ))
    servicemanager.Debugging(True)
    servicemanager.PrepareToHostSingle(cls)
    g_debugService = cls(argv)
    # Setup a ctrl+c handler to simulate a "stop"
    win32api.SetConsoleCtrlHandler(_DebugCtrlHandler, True)
    try:
        g_debugService.SvcRun()
    finally:
        win32api.SetConsoleCtrlHandler(_DebugCtrlHandler, False)
        servicemanager.Debugging(False)
        g_debugService = None


def GetServiceClassString(cls, argv=None):
    if argv is None:
        argv = sys.argv
    import pickle
    modName = pickle.whichmodule(cls, cls.__name__)
    if modName == '__main__':
        try:
            fname = win32api.GetFullPathName(argv[0])
            path = os.path.split(fname)[0]
            # Eaaaahhhh - sometimes this will be a short filename, which causes
            # problems with 1.5.1 and the silly filename case rule.
            # Get the long name
            fname = os.path.join(path, win32api.FindFiles(fname)[0][8])
        except win32api.error:
            raise error("Could not resolve the path name '%s' to a full path" %
                        (argv[0]))
        modName = os.path.splitext(fname)[0]
    return modName + "." + cls.__name__


def QueryServiceStatus(serviceName, machine=None):
    hscm = win32service.OpenSCManager(machine, None,
                                      win32service.SC_MANAGER_CONNECT)
    try:

        hs = SmartOpenService(hscm, serviceName,
                              win32service.SERVICE_QUERY_STATUS)
        try:
            status = win32service.QueryServiceStatus(hs)
        finally:
            win32service.CloseServiceHandle(hs)
    finally:
        win32service.CloseServiceHandle(hscm)
    return status


def usage():
    try:
        fname = os.path.split(sys.argv[0])[1]
    except:
        fname = sys.argv[0]
    print(
        "Usage: '%s [options] install|update|remove|start [...]|stop|restart [...]|debug [...]'"
        % fname)
    print("Options for 'install' and 'update' commands only:")
    print(
        " --username domain\\username : The Username the service is to run under"
    )
    print(" --password password : The password for the username")
    print(
        " --startup [manual|auto|disabled|delayed] : How the service starts, default = manual"
    )
    print(" --interactive : Allow the service to interact with the desktop.")
    print(
        " --perfmonini file: .ini file to use for registering performance monitor data"
    )
    print(" --perfmondll file: .dll file to use when querying the service for")
    print("   performance data, default = perfmondata.dll")
    print("Options for 'start' and 'stop' commands only:")
    print(" --wait seconds: Wait for the service to actually start or stop.")
    print(
        "                 If you specify --wait with the 'stop' option, the service"
    )
    print(
        "                 and all dependent services will be stopped, each waiting"
    )
    print("                 the specified period.")
    sys.exit(1)


def HandleCommandLine(cls,
                      serviceClassString=None,
                      argv=None,
                      customInstallOptions="",
                      customOptionHandler=None):
    """Utility function allowing services to process the command line.

    Allows standard commands such as 'start', 'stop', 'debug', 'install' etc.

    Install supports 'standard' command line options prefixed with '--', such as
    --username, --password, etc.  In addition,
    the function allows custom command line options to be handled by the calling function.
    """
    err = 0

    if argv is None:
        argv = sys.argv

    if len(argv) <= 1:
        usage()

    serviceName = cls._svc_name_
    serviceDisplayName = cls._svc_display_name_
    if serviceClassString is None:
        serviceClassString = GetServiceClassString(cls)

    # Pull apart the command line
    import getopt
    try:
        opts, args = getopt.getopt(argv[1:], customInstallOptions, [
            "password=", "username=", "startup=", "perfmonini=", "perfmondll=",
            "interactive", "wait="
        ])
    except getopt.error as details:
        print(details)
        usage()
    userName = None
    password = None
    perfMonIni = perfMonDll = None
    startup = None
    delayedstart = None
    interactive = None
    waitSecs = 0
    for opt, val in opts:
        if opt == '--username':
            userName = val
        elif opt == '--password':
            password = val
        elif opt == '--perfmonini':
            perfMonIni = val
        elif opt == '--perfmondll':
            perfMonDll = val
        elif opt == '--interactive':
            interactive = 1
        elif opt == '--startup':
            map = {
                "manual": win32service.SERVICE_DEMAND_START,
                "auto": win32service.SERVICE_AUTO_START,
                "delayed": win32service.
                SERVICE_AUTO_START,  # ChangeServiceConfig2 called later
                "disabled": win32service.SERVICE_DISABLED
            }
            try:
                startup = map[val.lower()]
            except KeyError:
                print("'%s' is not a valid startup option" % val)
            if val.lower() == "delayed":
                delayedstart = True
            elif val.lower() == "auto":
                delayedstart = False
            # else no change
        elif opt == '--wait':
            try:
                waitSecs = int(val)
            except ValueError:
                print("--wait must specify an integer number of seconds.")
                usage()

    arg = args[0]
    knownArg = 0
    # First we process all arguments which pass additional args on
    if arg == "start":
        knownArg = 1
        print("Starting service %s" % (serviceName))
        try:
            StartService(serviceName, args[1:])
            if waitSecs:
                WaitForServiceStatus(serviceName, win32service.SERVICE_RUNNING,
                                     waitSecs)
        except win32service.error as exc:
            print("Error starting service: %s" % exc.strerror)
            err = exc.winerror

    elif arg == "restart":
        knownArg = 1
        print("Restarting service %s" % (serviceName))
        RestartService(serviceName, args[1:])
        if waitSecs:
            WaitForServiceStatus(serviceName, win32service.SERVICE_RUNNING,
                                 waitSecs)

    elif arg == "debug":
        knownArg = 1
        if not hasattr(sys, "frozen"):
            # non-frozen services use pythonservice.exe which handles a
            # -debug option
            svcArgs = " ".join(args[1:])
            try:
                exeName = LocateSpecificServiceExe(serviceName)
            except win32api.error as exc:
                if exc.winerror == winerror.ERROR_FILE_NOT_FOUND:
                    print("The service does not appear to be installed.")
                    print("Please install the service before debugging it.")
                    sys.exit(1)
                raise
            try:
                os.system("%s -debug %s %s" % (exeName, serviceName, svcArgs))
            # ^C is used to kill the debug service.  Sometimes Python also gets
            # interrupted - ignore it...
            except KeyboardInterrupt:
                pass
        else:
            # py2exe services don't use pythonservice - so we simulate
            # debugging here.
            DebugService(cls, args)

    if not knownArg and len(args) != 1:
        usage()  # the rest of the cmds don't take addn args

    if arg == "install":
        knownArg = 1
        try:
            serviceDeps = cls._svc_deps_
        except AttributeError:
            serviceDeps = None
        try:
            exeName = cls._exe_name_
        except AttributeError:
            exeName = None  # Default to PythonService.exe
        try:
            exeArgs = cls._exe_args_
        except AttributeError:
            exeArgs = None
        try:
            description = cls._svc_description_
        except AttributeError:
            description = None
        print("Installing service %s" % (serviceName, ))
        # Note that we install the service before calling the custom option
        # handler, so if the custom handler fails, we have an installed service (from NT's POV)
        # but is unlikely to work, as the Python code controlling it failed.  Therefore
        # we remove the service if the first bit works, but the second doesnt!
        try:
            InstallService(
                serviceClassString,
                serviceName,
                serviceDisplayName,
                serviceDeps=serviceDeps,
                startType=startup,
                bRunInteractive=interactive,
                userName=userName,
                password=password,
                exeName=exeName,
                perfMonIni=perfMonIni,
                perfMonDll=perfMonDll,
                exeArgs=exeArgs,
                description=description,
                delayedstart=delayedstart)
            if customOptionHandler:
                customOptionHandler(*(opts, ))
            print("Service installed")
        except win32service.error as exc:
            if exc.winerror == winerror.ERROR_SERVICE_EXISTS:
                arg = "update"  # Fall through to the "update" param!
            else:
                print("Error installing service: %s (%d)" % (exc.strerror,
                                                             exc.winerror))
                err = exc.winerror
        except ValueError as msg:  # Can be raised by custom option handler.
            print("Error installing service: %s" % str(msg))
            err = -1
            # xxx - maybe I should remove after _any_ failed install - however,
            # xxx - it may be useful to help debug to leave the service as it failed.
            # xxx - We really _must_ remove as per the comments above...
            # As we failed here, remove the service, so the next installation
            # attempt works.
            try:
                RemoveService(serviceName)
            except win32api.error:
                print(
                    "Warning - could not remove the partially installed service."
                )

    if arg == "update":
        knownArg = 1
        try:
            serviceDeps = cls._svc_deps_
        except AttributeError:
            serviceDeps = None
        try:
            exeName = cls._exe_name_
        except AttributeError:
            exeName = None  # Default to PythonService.exe
        try:
            exeArgs = cls._exe_args_
        except AttributeError:
            exeArgs = None
        try:
            description = cls._svc_description_
        except AttributeError:
            description = None
        print("Changing service configuration")
        try:
            ChangeServiceConfig(
                serviceClassString,
                serviceName,
                serviceDeps=serviceDeps,
                startType=startup,
                bRunInteractive=interactive,
                userName=userName,
                password=password,
                exeName=exeName,
                displayName=serviceDisplayName,
                perfMonIni=perfMonIni,
                perfMonDll=perfMonDll,
                exeArgs=exeArgs,
                description=description,
                delayedstart=delayedstart)
            if customOptionHandler:
                customOptionHandler(*(opts, ))
            print("Service updated")
        except win32service.error as exc:
            print("Error changing service configuration: %s (%d)" %
                  (exc.strerror, exc.winerror))
            err = exc.winerror

    elif arg == "remove":
        knownArg = 1
        print("Removing service %s" % (serviceName))
        try:
            RemoveService(serviceName)
            print("Service removed")
        except win32service.error as exc:
            print("Error removing service: %s (%d)" % (exc.strerror,
                                                       exc.winerror))
            err = exc.winerror
    elif arg == "stop":
        knownArg = 1
        print("Stopping service %s" % (serviceName))
        try:
            if waitSecs:
                StopServiceWithDeps(serviceName, waitSecs=waitSecs)
            else:
                StopService(serviceName)
        except win32service.error as exc:
            print("Error stopping service: %s (%d)" % (exc.strerror,
                                                       exc.winerror))
            err = exc.winerror
    if not knownArg:
        err = -1
        print("Unknown command - '%s'" % arg)
        usage()
    return err


#
# Useful base class to build services from.
#


class ServiceFramework:
    # Required Attributes:
    # _svc_name_ = The service name
    # _svc_display_name_ = The service display name

    # Optional Attributes:
    _svc_deps_ = None  # sequence of service names on which this depends
    _exe_name_ = None  # Default to PythonService.exe
    _exe_args_ = None  # Default to no arguments
    _svc_description_ = None  # Only exists on Windows 2000 or later, ignored on windows NT

    def __init__(self, args):
        import servicemanager
        self.ssh = servicemanager.RegisterServiceCtrlHandler(
            args[0], self.ServiceCtrlHandlerEx, True)
        servicemanager.SetEventSourceName(self._svc_name_)
        self.checkPoint = 0

    def GetAcceptedControls(self):
        # Setup the service controls we accept based on our attributes. Note
        # that if you need to handle controls via SvcOther[Ex](), you must
        # override this.
        accepted = 0
        if hasattr(self, "SvcStop"):
            accepted = accepted | win32service.SERVICE_ACCEPT_STOP
        if hasattr(self, "SvcPause") and hasattr(self, "SvcContinue"):
            accepted = accepted | win32service.SERVICE_ACCEPT_PAUSE_CONTINUE
        if hasattr(self, "SvcShutdown"):
            accepted = accepted | win32service.SERVICE_ACCEPT_SHUTDOWN
        return accepted

    def ReportServiceStatus(self,
                            serviceStatus,
                            waitHint=5000,
                            win32ExitCode=0,
                            svcExitCode=0):
        if self.ssh is None:  # Debugging!
            return
        if serviceStatus == win32service.SERVICE_START_PENDING:
            accepted = 0
        else:
            accepted = self.GetAcceptedControls()

        if serviceStatus in [
                win32service.SERVICE_RUNNING, win32service.SERVICE_STOPPED
        ]:
            checkPoint = 0
        else:
            self.checkPoint = self.checkPoint + 1
            checkPoint = self.checkPoint

        # Now report the status to the control manager
        status = (
            win32service.SERVICE_WIN32_OWN_PROCESS,
            serviceStatus,
            accepted,  # dwControlsAccepted,
            win32ExitCode,  # dwWin32ExitCode;
            svcExitCode,  # dwServiceSpecificExitCode;
            checkPoint,  # dwCheckPoint;
            waitHint)
        win32service.SetServiceStatus(self.ssh, status)

    def SvcInterrogate(self):
        # Assume we are running, and everyone is happy.
        self.ReportServiceStatus(win32service.SERVICE_RUNNING)

    def SvcOther(self, control):
        try:
            print("Unknown control status - %d" % control)
        except IOError:
            # services may not have a valid stdout!
            pass

    def ServiceCtrlHandler(self, control):
        return self.ServiceCtrlHandlerEx(control, 0, None)

    # The 'Ex' functions, which take additional params
    def SvcOtherEx(self, control, event_type, data):
        # The default here is to call self.SvcOther as that is the old behaviour.
        # If you want to take advantage of the extra data, override this method
        return self.SvcOther(control)

    def ServiceCtrlHandlerEx(self, control, event_type, data):
        if control == win32service.SERVICE_CONTROL_STOP:
            return self.SvcStop()
        elif control == win32service.SERVICE_CONTROL_PAUSE:
            return self.SvcPause()
        elif control == win32service.SERVICE_CONTROL_CONTINUE:
            return self.SvcContinue()
        elif control == win32service.SERVICE_CONTROL_INTERROGATE:
            return self.SvcInterrogate()
        elif control == win32service.SERVICE_CONTROL_SHUTDOWN:
            return self.SvcShutdown()
        else:
            return self.SvcOtherEx(control, event_type, data)

    def SvcRun(self):
        self.ReportServiceStatus(win32service.SERVICE_RUNNING)
        self.SvcDoRun()
        # Once SvcDoRun terminates, the service has stopped.
        # We tell the SCM the service is still stopping - the C framework
        # will automatically tell the SCM it has stopped when this returns.
        self.ReportServiceStatus(win32service.SERVICE_STOP_PENDING)



import pytest
import pytest
from win32crypt import *
@pytest.mark.xfail
def test_CryptProtectData():
    CryptProtectData()     



@pytest.mark.xfail
def test_CryptUnprotectData():
    CryptUnprotectData()     





import os
import site


def list_files(startpath):
    for root, dirs, files in os.walk(startpath):
        level = root.replace(startpath, '').count(os.sep)
        indent = ' ' * 4 * (level)
        print(('{}{}/'.format(indent, os.path.basename(root))))
        subindent = ' ' * 4 * (level + 1)
        for f in files:
            print(('{}{}'.format(subindent, f)))


def main():
    for site_packages in site.getsitepackages():
        list_files(site_packages)


if __name__ == '__main__':
    main()



import string

from setuptools import Extension


class WinExt(Extension):
    # Base class for all win32 extensions, with some predefined
    # library and include dirs, and predefined windows libraries.
    # Additionally a method to parse .def files into lists of exported
    # symbols, and to read

    def __init__(
            self,
            name,
            sources,
            include_dirs=[],
            define_macros=None,
            undef_macros=None,
            library_dirs=[],
            libraries=[],
            runtime_library_dirs=None,
            extra_objects=None,
            extra_compile_args=None,
            extra_link_args=None,
            export_symbols=None,
            export_symbol_file=None,
            pch_header=None,
            windows_h_version=None,  # min version of windows.h needed.
            extra_swig_commands=None,
            is_regular_dll=False,  # regular Windows DLL?
            # list of headers which may not be installed forcing us to
            # skip this extension
            optional_headers=[],
            depends=None,
            platforms=None,  # none means 'all platforms'
            unicode_mode=None,
            # 'none'==default or specifically true/false.
            implib_name=None,
            delay_load_libraries='',
            external_modules=[],  # Libs that needed for building
            build_temp_library_dirs=[]):

        self.delay_load_libraries = delay_load_libraries.split()
        libraries.extend(self.delay_load_libraries)

        if export_symbol_file:
            export_symbols = export_symbols or []
            export_symbols.extend(self.parse_def_file(export_symbol_file))

        # Some of our swigged files behave differently in distutils vs
        # MSVC based builds.  Always define DISTUTILS_BUILD so they can tell.
        define_macros = define_macros or []
        define_macros.append(('DISTUTILS_BUILD', None))
        define_macros.append(('_CRT_SECURE_NO_WARNINGS', None))

        self.pch_header = pch_header
        self.extra_swig_commands = extra_swig_commands or []
        self.windows_h_version = windows_h_version
        self.optional_headers = optional_headers
        self.is_regular_dll = is_regular_dll
        self.platforms = platforms
        self.implib_name = implib_name
        self.external_modules = external_modules
        self.build_temp_library_dirs = build_temp_library_dirs
        Extension.__init__(self, name, sources, include_dirs, define_macros,
                           undef_macros, library_dirs, libraries,
                           runtime_library_dirs, extra_objects,
                           extra_compile_args, extra_link_args, export_symbols)

        if not hasattr(self, 'swig_deps'):
            self.swig_deps = []
        self.extra_compile_args.extend(['/DUNICODE', '/D_UNICODE', '/DWINNT'])
        self.unicode_mode = unicode_mode

        if self.delay_load_libraries:
            self.libraries.append('delayimp')
            for delay_lib in self.delay_load_libraries:
                self.extra_link_args.append('/delayload:%s.dll' % delay_lib)

        if not hasattr(self, '_needs_stub'):
            self._needs_stub = False

    def parse_def_file(self, path):
        # Extract symbols to export from a def-file
        result = []
        for line in open(path).readlines():
            line = line.rstrip()
            if line and line[0] in string.whitespace:
                tokens = line.split()
                if not tokens[0][0] in string.ascii_letters:
                    continue
                result.append(','.join(tokens))
        return result



import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32wnet import *
def test_NCB():
    NCB()     



@pytest.mark.xfail
def test_NCBBuffer():
    NCBBuffer()     



def test_NETRESOURCE():
    NETRESOURCE()     



@pytest.mark.xfail
def test_Netbios():
    Netbios()     



@pytest.mark.xfail
def test_WNetAddConnection2():
    WNetAddConnection2()     



@pytest.mark.xfail
def test_WNetAddConnection3():
    WNetAddConnection3()     



@pytest.mark.xfail
def test_WNetCancelConnection2():
    WNetCancelConnection2()     



@pytest.mark.xfail
def test_WNetCloseEnum():
    WNetCloseEnum()     



@pytest.mark.xfail
def test_WNetEnumResource():
    WNetEnumResource()     



def test_WNetGetLastError():
    WNetGetLastError()     



@pytest.mark.xfail
def test_WNetGetResourceInformation():
    WNetGetResourceInformation()     



@pytest.mark.xfail
def test_WNetGetResourceParent():
    WNetGetResourceParent()     



@pytest.mark.xfail
def test_WNetGetUniversalName():
    WNetGetUniversalName()     



def test_WNetGetUser():
    WNetGetUser()     



@pytest.mark.xfail
def test_WNetOpenEnum():
    WNetOpenEnum()     





import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32process import *


@pytest.mark.xfail
def test_AttachThreadInput():
    AttachThreadInput()


@pytest.mark.xfail
def test_CreateProcess():
    CreateProcess()


@pytest.mark.xfail
def test_CreateProcessAsUser():
    CreateProcessAsUser()


@pytest.mark.xfail
def test_CreateRemoteThread():
    CreateRemoteThread()


@pytest.mark.xfail
def test_EnumProcessModules():
    EnumProcessModules()


def test_EnumProcesses():
    EnumProcesses()


@pytest.mark.xfail
def test_ExitProcess():
    ExitProcess()


def test_GetCurrentProcess():
    GetCurrentProcess()


def test_GetCurrentProcessId():
    GetCurrentProcessId()


@pytest.mark.xfail
def test_GetExitCodeProcess():
    GetExitCodeProcess()


@pytest.mark.xfail
def test_GetExitCodeThread():
    GetExitCodeThread()


@pytest.mark.xfail
def test_GetGuiResources():
    GetGuiResources()


@pytest.mark.xfail
def test_GetModuleFileNameEx():
    GetModuleFileNameEx()


@pytest.mark.xfail
def test_GetPriorityClass():
    GetPriorityClass()


@pytest.mark.xfail
def test_GetProcessAffinityMask():
    GetProcessAffinityMask()


@pytest.mark.xfail
def test_GetProcessId():
    GetProcessId()


@pytest.mark.xfail
def test_GetProcessIoCounters():
    GetProcessIoCounters()


@pytest.mark.xfail
def test_GetProcessMemoryInfo():
    GetProcessMemoryInfo()


@pytest.mark.xfail
def test_GetProcessPriorityBoost():
    GetProcessPriorityBoost()


def test_GetProcessShutdownParameters():
    GetProcessShutdownParameters()


@pytest.mark.xfail
def test_GetProcessTimes():
    GetProcessTimes()


@pytest.mark.xfail
def test_GetProcessVersion():
    GetProcessVersion()


def test_GetProcessWindowStation():
    GetProcessWindowStation()


@pytest.mark.xfail
def test_GetProcessWorkingSetSize():
    GetProcessWorkingSetSize()


def test_GetStartupInfo():
    GetStartupInfo()


@pytest.mark.xfail
def test_GetThreadIOPendingFlag():
    GetThreadIOPendingFlag()


@pytest.mark.xfail
def test_GetThreadPriority():
    GetThreadPriority()


@pytest.mark.xfail
def test_GetThreadPriorityBoost():
    GetThreadPriorityBoost()


@pytest.mark.xfail
def test_GetThreadTimes():
    GetThreadTimes()


@pytest.mark.xfail
def test_GetWindowThreadProcessId():
    GetWindowThreadProcessId()


def test_IsWow64Process():
    IsWow64Process()


@pytest.mark.xfail
def test_ResumeThread():
    ResumeThread()


def test_STARTUPINFO():
    STARTUPINFO()


@pytest.mark.xfail
def test_SetPriorityClass():
    SetPriorityClass()


@pytest.mark.xfail
def test_SetProcessAffinityMask():
    SetProcessAffinityMask()


@pytest.mark.xfail
def test_SetProcessPriorityBoost():
    SetProcessPriorityBoost()


@pytest.mark.xfail
def test_SetProcessShutdownParameters():
    SetProcessShutdownParameters()


@pytest.mark.xfail
def test_SetProcessWorkingSetSize():
    SetProcessWorkingSetSize()


@pytest.mark.xfail
def test_SetThreadAffinityMask():
    SetThreadAffinityMask()


@pytest.mark.xfail
def test_SetThreadIdealProcessor():
    SetThreadIdealProcessor()


@pytest.mark.xfail
def test_SetThreadPriority():
    SetThreadPriority()


@pytest.mark.xfail
def test_SetThreadPriorityBoost():
    SetThreadPriorityBoost()


@pytest.mark.xfail
def test_SuspendThread():
    SuspendThread()


@pytest.mark.xfail
def test_TerminateProcess():
    TerminateProcess()


@pytest.mark.xfail
def test_beginthreadex():
    beginthreadex()



import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32clipboard import *
@pytest.mark.xfail
def test_ChangeClipboardChain():
    ChangeClipboardChain()     



@pytest.mark.xfail
def test_CloseClipboard():
    CloseClipboard()     



@pytest.mark.xfail
def test_CountClipboardFormats():
    CountClipboardFormats()     



@pytest.mark.xfail
def test_EmptyClipboard():
    EmptyClipboard()     



@pytest.mark.xfail
def test_EnumClipboardFormats():
    EnumClipboardFormats()     



@pytest.mark.xfail
def test_GetClipboardData():
    GetClipboardData()     



@pytest.mark.xfail
def test_GetClipboardDataHandle():
    GetClipboardDataHandle()     



@pytest.mark.xfail
def test_GetClipboardFormatName():
    GetClipboardFormatName()     



@pytest.mark.xfail
def test_GetClipboardOwner():
    GetClipboardOwner()     



def test_GetClipboardSequenceNumber():
    GetClipboardSequenceNumber()     



@pytest.mark.xfail
def test_GetClipboardViewer():
    GetClipboardViewer()     



@pytest.mark.xfail
def test_GetGlobalMemory():
    GetGlobalMemory()     



@pytest.mark.xfail
def test_GetOpenClipboardWindow():
    GetOpenClipboardWindow()     



@pytest.mark.xfail
def test_GetPriorityClipboardFormat():
    GetPriorityClipboardFormat()     



@pytest.mark.xfail
def test_IsClipboardFormatAvailable():
    IsClipboardFormatAvailable()     



def test_OpenClipboard():
    OpenClipboard()     



@pytest.mark.xfail
def test_RegisterClipboardFormat():
    RegisterClipboardFormat()     



@pytest.mark.xfail
def test_SetClipboardData():
    SetClipboardData()     



@pytest.mark.xfail
def test_SetClipboardText():
    SetClipboardText()     



@pytest.mark.xfail
def test_SetClipboardViewer():
    SetClipboardViewer()     





import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
import pytest
from win32print import *
@pytest.mark.xfail
def test_AbortDoc():
    AbortDoc()     



@pytest.mark.xfail
def test_AbortPrinter():
    AbortPrinter()     



@pytest.mark.xfail
def test_AddForm():
    AddForm()     



@pytest.mark.xfail
def test_AddJob():
    AddJob()     



@pytest.mark.xfail
def test_AddPrinter():
    AddPrinter()     



@pytest.mark.xfail
def test_AddPrinterConnection():
    AddPrinterConnection()     



@pytest.mark.xfail
def test_ClosePrinter():
    ClosePrinter()     



@pytest.mark.xfail
def test_DeleteForm():
    DeleteForm()     



@pytest.mark.xfail
def test_DeletePrinter():
    DeletePrinter()     



@pytest.mark.xfail
def test_DeletePrinterConnection():
    DeletePrinterConnection()     



@pytest.mark.xfail
def test_DeletePrinterDriver():
    DeletePrinterDriver()     



@pytest.mark.xfail
def test_DeletePrinterDriverEx():
    DeletePrinterDriverEx()     



@pytest.mark.xfail
def test_DeviceCapabilities():
    DeviceCapabilities()     



@pytest.mark.xfail
def test_DocumentProperties():
    DocumentProperties()     



@pytest.mark.xfail
def test_EndDoc():
    EndDoc()     



@pytest.mark.xfail
def test_EndDocPrinter():
    EndDocPrinter()     



@pytest.mark.xfail
def test_EndPage():
    EndPage()     



@pytest.mark.xfail
def test_EndPagePrinter():
    EndPagePrinter()     



@pytest.mark.xfail
def test_EnumForms():
    EnumForms()     



@pytest.mark.xfail
def test_EnumJobs():
    EnumJobs()     



@pytest.mark.xfail
def test_EnumMonitors():
    EnumMonitors()     



@pytest.mark.xfail
def test_EnumPorts():
    EnumPorts()     



@pytest.mark.xfail
def test_EnumPrintProcessorDatatypes():
    EnumPrintProcessorDatatypes()     



@pytest.mark.xfail
def test_EnumPrintProcessors():
    EnumPrintProcessors()     



@pytest.mark.xfail
def test_EnumPrinterDrivers():
    EnumPrinterDrivers()     



@pytest.mark.xfail
def test_EnumPrinters():
    EnumPrinters()     



@pytest.mark.xfail
def test_FlushPrinter():
    FlushPrinter()     



def test_GetDefaultPrinter():
    GetDefaultPrinter()     



def test_GetDefaultPrinterW():
    GetDefaultPrinterW()     



@pytest.mark.xfail
def test_GetDeviceCaps():
    GetDeviceCaps()     



@pytest.mark.xfail
def test_GetForm():
    GetForm()     



@pytest.mark.xfail
def test_GetJob():
    GetJob()     



def test_GetPrintProcessorDirectory():
    GetPrintProcessorDirectory()     



@pytest.mark.xfail
def test_GetPrinter():
    GetPrinter()     



def test_GetPrinterDriverDirectory():
    GetPrinterDriverDirectory()     



@pytest.mark.xfail
def test_OpenPrinter():
    OpenPrinter()     



@pytest.mark.xfail
def test_ScheduleJob():
    ScheduleJob()     



@pytest.mark.xfail
def test_SetDefaultPrinter():
    SetDefaultPrinter()     



@pytest.mark.xfail
def test_SetDefaultPrinterW():
    SetDefaultPrinterW()     



@pytest.mark.xfail
def test_SetForm():
    SetForm()     



@pytest.mark.xfail
def test_SetJob():
    SetJob()     



@pytest.mark.xfail
def test_SetPrinter():
    SetPrinter()     



@pytest.mark.xfail
def test_StartDoc():
    StartDoc()     



@pytest.mark.xfail
def test_StartDocPrinter():
    StartDocPrinter()     



@pytest.mark.xfail
def test_StartPage():
    StartPage()     



@pytest.mark.xfail
def test_StartPagePrinter():
    StartPagePrinter()     



@pytest.mark.xfail
def test_WritePrinter():
    WritePrinter()     


